@@ -22,9 +22,10 @@ Usage:
 """
 
 from .client import CodeToolsClient
+from .client_manager import ClientManager
 from .tools import ReadTool, WriteTool, EditTool, RunTool
 
 __version__ = "2.0.0"
 __author__ = "balajidinesh"
 
-__all__ = ["CodeToolsClient", "ReadTool", "WriteTool", "EditTool", "RunTool"]
\ No newline at end of file
+__all__ = ["CodeToolsClient", "ClientManager", "ReadTool", "WriteTool", "EditTool", "RunTool"]
\ No newline at end of file
