@@ -9,15 +9,20 @@ import json
 import time
 import subprocess
 import requests
+import urllib3
 from typing import Dict, Any, Optional, List, Callable, Union
 import os
 import logging
 import functools
+import re
+import tempfile
+import socket
+import random
+from urllib.parse import urlparse, urlunparse
 from datetime import datetime
 
 # Import tool classes to extract docstrings
 from aicodetools.tools import ReadTool, WriteTool, EditTool, RunTool
-from aicodetools.tools.read import LineArgs, RegexArgs
 
 
 # Tool names for tools() method
@@ -37,8 +42,8 @@ class CodeToolsClient:
 
         # Set connection pooling and keep-alive for better performance
         adapter = requests.adapters.HTTPAdapter(
-            pool_connections=1,
-            pool_maxsize=1,
+            pool_connections=10,
+            pool_maxsize=10,
             max_retries=requests.adapters.Retry(
                 total=3,
                 backoff_factor=1,
@@ -72,25 +77,125 @@ class CodeToolsClient:
         if auto_start:
             self._ensure_server_running()
 
+    @staticmethod
+    def _sanitize_image_name(image_name: str) -> str:
+        """
+        Sanitize Docker image name for use in derived image tags.
+
+        Converts: python:3.11-slim -> python-3.11-slim
+        Converts: myregistry.com/myimage:v1.2.3 -> myregistry-com-myimage-v1-2-3
+        """
+        # Replace non-alphanumeric characters with hyphens
+        sanitized = re.sub(r'[^a-zA-Z0-9.-]', '-', image_name)
+        # Remove leading/trailing hyphens
+        sanitized = sanitized.strip('-')
+        return sanitized.lower()
+
+    def _get_prebuilt_image_name(self) -> str:
+        """Get the name of the pre-built image for the current base image."""
+        sanitized_base = self._sanitize_image_name(self.docker_image)
+        return f"aicodetools-{sanitized_base}"
+
+    def _prebuilt_image_exists(self) -> bool:
+        """Check if pre-built image already exists."""
+        prebuilt_image = self._get_prebuilt_image_name()
+        try:
+            result = subprocess.run(
+                ['docker', 'images', '-q', prebuilt_image],
+                capture_output=True, text=True, timeout=10
+            )
+            return bool(result.stdout.strip())
+        except Exception as e:
+            logging.warning(f"Failed to check for pre-built image: {e}")
+            return False
+
+    def _build_prebuilt_image(self) -> bool:
+        """Build pre-built image with aicodetools installed."""
+        prebuilt_image = self._get_prebuilt_image_name()
+
+        logging.info(f"Building pre-built image '{prebuilt_image}' (one-time operation)...")
+
+        # Create Dockerfile content
+        dockerfile_content = f"""FROM {self.docker_image}
+RUN pip install --break-system-packages aicodetools
+CMD ["python", "-m", "aicodetools.server", "--host", "0.0.0.0", "--port", "8080"]
+"""
+
+        try:
+            # Create temporary directory for build context
+            with tempfile.TemporaryDirectory() as tmpdir:
+                dockerfile_path = os.path.join(tmpdir, 'Dockerfile')
+
+                # Write Dockerfile
+                with open(dockerfile_path, 'w') as f:
+                    f.write(dockerfile_content)
+
+                # Build image
+                build_cmd = [
+                    'docker', 'build',
+                    '-t', prebuilt_image,
+                    '-f', dockerfile_path,
+                    tmpdir
+                ]
+
+                logging.info(f"Running: docker build -t {prebuilt_image} ...")
+                result = subprocess.run(
+                    build_cmd,
+                    capture_output=True,
+                    text=True,
+                    timeout=300  # 5 minute timeout for build
+                )
+
+                if result.returncode != 0:
+                    logging.error(f"Failed to build pre-built image: {result.stderr}")
+                    return False
+
+                logging.info(f"Successfully built pre-built image '{prebuilt_image}'")
+                return True
+
+        except Exception as e:
+            logging.error(f"Error building pre-built image: {e}")
+            return False
+
     def _ensure_server_running(self) -> bool:
         """Ensure the server is running, start if necessary."""
-        try:
-            # Check if server is already running
-            response = self.session.get(f"{self.server_url}/api/status", timeout=2)
-            if response.status_code == 200:
-                logging.info("Server is already running")
-                return True
-        except requests.exceptions.RequestException:
-            pass
+        # Suppress urllib3 warnings during connection checks (errors are expected if server not running)
+        urllib3_logger = logging.getLogger('urllib3.connectionpool')
+        original_urllib3_level = urllib3_logger.level
+        urllib3_logger.setLevel(logging.ERROR)
 
-        # Try to start server using Docker
-        return self._start_docker_server()
+        try:
+            # Try to connect to existing server with retries
+            for attempt in range(3):
+                try:
+                    response = self.session.get(f"{self.server_url}/api/status", timeout=2)
+                    if response.status_code == 200:
+                        logging.info("Server is already running")
+                        return True
+                except requests.exceptions.ConnectionError:
+                    # Connection refused - server not running
+                    if attempt < 2:
+                        time.sleep(0.5)  # Brief delay before retry
+                        continue
+                    break  # Server definitely not running
+                except requests.exceptions.RequestException:
+                    pass
+
+            # Try to start server using Docker
+            return self._start_docker_server()
+        finally:
+            # Re-enable urllib3 warnings
+            urllib3_logger.setLevel(original_urllib3_level)
 
     def _start_docker_server(self) -> bool:
         """Start the server using Docker."""
         try:
             logging.info("Starting AI Code Tools server in Docker...")
 
+            # Ensure we have an available port before attempting to start
+            self._ensure_available_port()
+            self._sync_server_url_with_port()
+
             # Check if Docker service is running
             try:
                 result = subprocess.run(['docker', 'info'], capture_output=True, text=True, timeout=10)
@@ -101,66 +206,202 @@ class CodeToolsClient:
                 logging.error("Docker not found or not running. Please install and start Docker.")
                 return False
 
-            # Check if Docker image exists
-            check_result = subprocess.run(['docker', 'images', '-q', self.docker_image],
-                                        capture_output=True, text=True, timeout=10)
+            # Determine which image to use
+            image_to_use = self.docker_image
+            use_prebuilt = False
 
-            if not check_result.stdout.strip():
-                # Image doesn't exist - try to pull it
-                logging.info(f"Pulling image: docker pull {self.docker_image}")
-                pull_result = subprocess.run(['docker', 'pull', self.docker_image],
-                                           capture_output=True, text=True, timeout=120)
-                if pull_result.returncode != 0:
-                    logging.error(f"Image '{self.docker_image}' not found. Please ensure it exists or use a standard Python image like 'python:3.11-slim'.")
-                    return False
+            # Check if pre-built image exists
+            if self._prebuilt_image_exists():
+                image_to_use = self._get_prebuilt_image_name()
+                use_prebuilt = True
+                logging.info(f"Using pre-built image: {image_to_use}")
+            else:
+                # Check if base image exists, pull if needed
+                check_result = subprocess.run(['docker', 'images', '-q', self.docker_image],
+                                            capture_output=True, text=True, timeout=10)
+
+                if not check_result.stdout.strip():
+                    # Base image doesn't exist - try to pull it
+                    logging.info(f"Pulling base image: {self.docker_image}")
+                    pull_result = subprocess.run(['docker', 'pull', self.docker_image],
+                                               capture_output=True, text=True, timeout=120)
+                    if pull_result.returncode != 0:
+                        logging.error(f"Image '{self.docker_image}' not found. Please ensure it exists or use a standard Python image like 'python:3.11-slim'.")
+                        return False
+
+                # Build pre-built image for future use
+                if not self._build_prebuilt_image():
+                    logging.warning("Failed to build pre-built image, falling back to on-demand installation")
+                    use_prebuilt = False
+                else:
+                    image_to_use = self._get_prebuilt_image_name()
+                    use_prebuilt = True
 
             # Clean up existing container
             subprocess.run(['docker', 'stop', self.container_name], capture_output=True, text=True, timeout=10)
             subprocess.run(['docker', 'rm', self.container_name], capture_output=True, text=True, timeout=10)
 
-            # Start container with pip-installed package
-            cmd_string = f'pip install --break-system-packages aicodetools && python -m aicodetools.server --host 0.0.0.0 --port 8080'
-            run_cmd = [
-                'docker', 'run', '-d', '--name', self.container_name,
-                '-p', f'{self.port}:8080',
-                '--rm',  # Auto-remove container when stopped
-                self.docker_image,
-                'bash', '-c', cmd_string
-            ]
+            # Start container with silent port retry on conflicts
+            max_port_attempts = 10
+            attempt = 0
+            while attempt < max_port_attempts:
+                if use_prebuilt:
+                    # Pre-built image already has aicodetools installed
+                    run_cmd = [
+                        'docker', 'run', '-d', '--name', self.container_name,
+                        '-p', f'{self.port}:8080',
+                        '--rm',  # Auto-remove container when stopped
+                        image_to_use
+                    ]
+                    exact_cmd = f"docker run -d --name {self.container_name} -p {self.port}:8080 --rm {image_to_use}"
+                else:
+                    # Fall back to on-demand installation
+                    cmd_string = f'pip install --break-system-packages aicodetools && python -m aicodetools.server --host 0.0.0.0 --port 8080'
+                    run_cmd = [
+                        'docker', 'run', '-d', '--name', self.container_name,
+                        '-p', f'{self.port}:8080',
+                        '--rm',  # Auto-remove container when stopped
+                        image_to_use,
+                        'bash', '-c', cmd_string
+                    ]
+                    exact_cmd = f"docker run -d --name {self.container_name} -p {self.port}:8080 --rm {image_to_use} bash -c \"{cmd_string}\""
 
-            # Show exact command for debugging
-            exact_cmd = f"docker run -d --name {self.container_name} -p {self.port}:8080 --rm {self.docker_image} bash -c \"{cmd_string}\""
-            logging.info(f"Starting container: {exact_cmd}")
-            run_result = subprocess.run(run_cmd, capture_output=True, text=True, timeout=120)
-            if run_result.returncode != 0:
-                if "port is already allocated" in run_result.stderr:
-                    logging.error(f"Port {self.port} is in use. Please stop other services or use a different port.")
-                elif "permission denied" in run_result.stderr:
+                logging.info(f"Starting container: {exact_cmd}")
+                run_result = subprocess.run(run_cmd, capture_output=True, text=True, timeout=120)
+                if run_result.returncode == 0:
+                    break
+
+                stderr = (run_result.stderr or '').lower()
+                if "port is already allocated" in stderr or "address already in use" in stderr:
+                    # Pick a new random available port silently and retry
+                    logging.debug(f"Port {self.port} in use; retrying with a random port")
+                    self.port = self._pick_random_available_port()
+                    self._sync_server_url_with_port()
+                    attempt += 1
+                    # Small delay before retry to avoid tight loop
+                    time.sleep(0.2)
+                    continue
+                elif "permission denied" in stderr:
                     logging.error("Docker permission denied. Try running as administrator or add user to docker group.")
+                    return False
                 else:
                     logging.error(f"Container failed to start: {run_result.stderr}")
+                    return False
+
+            if attempt >= max_port_attempts:
+                logging.error("Failed to find an available port after multiple attempts")
                 return False
 
             self.docker_container = self.container_name
 
-            # Wait for server to start
-            for _ in range(30):  # Wait up to 30 seconds
-                try:
-                    response = self.session.get(f"{self.server_url}/api/status", timeout=2)
-                    if response.status_code == 200:
-                        logging.info("Server started successfully")
-                        return True
-                except requests.exceptions.RequestException:
-                    pass
-                time.sleep(1)
+            # Add initial delay to let server start
+            # Pre-built images start faster, but still need a moment
+            initial_delay = 3 if use_prebuilt else 8
+            logging.info(f"Waiting {initial_delay} seconds for server to initialize...")
+            time.sleep(initial_delay)
 
-            logging.error("Server failed to start within timeout")
-            return False
+            # Suppress urllib3 warnings during startup (connection errors are expected)
+            urllib3_logger = logging.getLogger('urllib3.connectionpool')
+            original_urllib3_level = urllib3_logger.level
+            urllib3_logger.setLevel(logging.ERROR)
+
+            # Wait for server to start with retry logic
+            connection_attempts = 0
+            max_connection_attempts = 3
+
+            try:
+                for i in range(30):  # Wait up to 30 seconds
+                    try:
+                        response = self.session.get(f"{self.server_url}/api/status", timeout=2)
+                        if response.status_code == 200:
+                            logging.info("Server started successfully")
+                            # Re-enable urllib3 warnings
+                            urllib3_logger.setLevel(original_urllib3_level)
+                            return True
+                    except requests.exceptions.ConnectionError:
+                        # Connection refused - server not ready yet or failed
+                        connection_attempts += 1
+
+                        # After several connection refused, check if container is still running
+                        if connection_attempts > max_connection_attempts and i > 10:
+                            check_result = subprocess.run(
+                                ['docker', 'ps', '--filter', f'name={self.container_name}', '--format', '{{.Names}}'],
+                                capture_output=True, text=True, timeout=5
+                            )
+                            if self.container_name not in check_result.stdout:
+                                # Container crashed - get logs
+                                logs_result = subprocess.run(
+                                    ['docker', 'logs', self.container_name],
+                                    capture_output=True, text=True, timeout=5
+                                )
+                                logging.error(f"Container crashed. Logs:\n{logs_result.stdout}\n{logs_result.stderr}")
+                                return False
+                    except requests.exceptions.RequestException:
+                        pass
+                    time.sleep(1)
+
+                # Timeout reached - check container status
+                check_result = subprocess.run(
+                    ['docker', 'ps', '--filter', f'name={self.container_name}', '--format', '{{.Names}}'],
+                    capture_output=True, text=True, timeout=5
+                )
+                if self.container_name in check_result.stdout:
+                    logging.error("Server container is running but not responding to requests")
+                else:
+                    logging.error("Server container failed to start or crashed")
+
+                return False
+            finally:
+                # Always re-enable urllib3 warnings
+                urllib3_logger.setLevel(original_urllib3_level)
 
         except Exception as e:
             logging.error(f"Failed to start Docker server: {e}")
             return False
 
+    def _is_port_available(self, port: int) -> bool:
+        """Check if a localhost TCP port is available (not accepting connections)."""
+        try:
+            with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as sock:
+                sock.settimeout(0.5)
+                result = sock.connect_ex(('localhost', port))
+                return result != 0
+        except Exception:
+            return False
+
+    def _pick_random_available_port(self, min_port: int = 18080, max_port: int = 65535, max_tries: int = 200) -> int:
+        """Pick a random available port within range, avoiding obvious conflicts."""
+        for _ in range(max_tries):
+            port = random.randint(min_port, max_port)
+            if self._is_port_available(port):
+                return port
+        # Fallback: linear scan from min_port
+        for port in range(min_port, max_port + 1):
+            if self._is_port_available(port):
+                return port
+        raise RuntimeError("No available port found")
+
+    def _ensure_available_port(self) -> None:
+        """Ensure self.port is available; otherwise pick a random available one."""
+        try:
+            if not self._is_port_available(self.port):
+                self.port = self._pick_random_available_port()
+        except Exception:
+            # As a last resort, just pick a random port and proceed
+            self.port = random.randint(20000, 65000)
+
+    def _sync_server_url_with_port(self) -> None:
+        """Update self.server_url to use localhost with the current port."""
+        try:
+            parsed = urlparse(self.server_url)
+            # Always target localhost for Docker-run server
+            netloc = f"localhost:{self.port}"
+            updated = parsed._replace(scheme=parsed.scheme or 'http', netloc=netloc)
+            self.server_url = urlunparse(updated)
+        except Exception:
+            # Fallback
+            self.server_url = f"http://localhost:{self.port}"
+
     def _truncate_string(self, text: str, max_length: int) -> str:
         """Truncate string to max_length, adding ellipsis if truncated."""
         if len(text) <= max_length:
@@ -360,8 +601,24 @@ class CodeToolsClient:
             error_result = {"success": False, "error": "Request timed out"}
             self._log_tool_result(endpoint, error_result)
             return error_result
-        except requests.exceptions.ConnectionError:
-            error_result = {"success": False, "error": "Failed to connect to server"}
+        except requests.exceptions.ConnectionError as e:
+            # Connection refused - try to provide helpful context
+            error_msg = "Failed to connect to server"
+            if "Connection refused" in str(e) or "ConnectionRefusedError" in str(e):
+                # Check if Docker container is still running
+                if self.docker_container:
+                    try:
+                        check_result = subprocess.run(
+                            ['docker', 'ps', '--filter', f'name={self.container_name}', '--format', '{{.Names}}'],
+                            capture_output=True, text=True, timeout=5
+                        )
+                        if self.container_name not in check_result.stdout:
+                            error_msg = "Server container has stopped or crashed"
+                        else:
+                            error_msg = "Server is not responding (connection refused)"
+                    except Exception:
+                        pass
+            error_result = {"success": False, "error": error_msg}
             self._log_tool_result(endpoint, error_result)
             return error_result
         except requests.exceptions.RequestException as e:
@@ -401,22 +658,17 @@ class CodeToolsClient:
 
     def _create_read_tool(self) -> Callable:
         """Create read tool function with docstring from tools/read.py"""
-        def read_file_tool(file_path: str, read_args: Optional[Union[LineArgs, RegexArgs]] = None) -> Dict[str, Any]:
+        def read_file_tool(file_path: str, lines_start: Optional[int] = None,
+                          lines_end: Optional[int] = None, regex: Optional[str] = None) -> Dict[str, Any]:
             data = {"file_path": file_path}
 
-            # Handle read_args parameter if provided
-            if read_args is not None:
-                if isinstance(read_args, RegexArgs):
-                    data["read_args"] = {"regex": read_args.regex}
-                    data["read_args_type"] = "RegexArgs"
-                elif isinstance(read_args, LineArgs):
-                    read_args_data = {"lines_start": read_args.lines_start}
-                    if read_args.lines_end is not None:
-                        read_args_data["lines_end"] = read_args.lines_end
-                    if read_args.complete_lines:
-                        read_args_data["complete_lines"] = read_args.complete_lines
-                    data["read_args"] = read_args_data
-                    data["read_args_type"] = "LineArgs"
+            # Add optional parameters if provided
+            if lines_start is not None:
+                data["lines_start"] = lines_start
+            if lines_end is not None:
+                data["lines_end"] = lines_end
+            if regex is not None:
+                data["regex"] = regex
 
             return self._make_request("read", data)
 
