@@ -15,7 +15,6 @@ from typing import Dict, Any
 import logging
 
 from aicodetools.tools import ReadTool, WriteTool, EditTool, RunTool
-from aicodetools.tools.read import LineArgs, RegexArgs
 
 
 class CodeToolsHandler(BaseHTTPRequestHandler):
@@ -106,7 +105,7 @@ class CodeToolsHandler(BaseHTTPRequestHandler):
             self._send_json_response(error_response, status_code=500)
 
     def _handle_read(self, data: Dict[str, Any]) -> Dict[str, Any]:
-        """Handle read tool requests with new simplified API."""
+        """Handle read tool requests with flattened parameters."""
         file_path = data.get('file_path')
         if not file_path:
             return {"success": False, "error": "file_path is required"}
@@ -115,36 +114,17 @@ class CodeToolsHandler(BaseHTTPRequestHandler):
         self.write_tool.mark_file_as_read(file_path)
         self.edit_tool.mark_file_as_read(file_path)
 
-        # Parse read arguments - handle both direct and nested formats
-        read_args = None
-        try:
-            # Check for nested read_args format (from client)
-            args_data = data.get('read_args', data)  # Fallback to data itself for backward compatibility
+        # Extract flattened parameters
+        lines_start = data.get('lines_start')
+        lines_end = data.get('lines_end')
+        regex = data.get('regex')
 
-            if args_data.get('regex'):
-                # RegexArgs mode
-                read_args = RegexArgs(regex=args_data['regex'])
-            elif args_data.get('lines_start'):
-                # LineArgs mode
-                read_args = LineArgs(
-                    lines_start=args_data['lines_start'],
-                    lines_end=args_data.get('lines_end'),
-                    complete_lines=args_data.get('complete_lines', False)
-                )
-            # If neither, read_args stays None for full file read
-
-            return self.read_tool.read_file(file_path, read_args)
-
-        except Exception as e:
-            return {
-                "success": False,
-                "error": f"Invalid read arguments: {str(e)}",
-                "suggestions": [
-                    "Use 'regex' for pattern matching",
-                    "Use 'lines_start' (and optionally 'lines_end') for line ranges",
-                    "Omit both for full file read"
-                ]
-            }
+        return self.read_tool.read_file(
+            file_path=file_path,
+            lines_start=lines_start,
+            lines_end=lines_end,
+            regex=regex
+        )
 
     def _handle_write(self, data: Dict[str, Any]) -> Dict[str, Any]:
         """Handle write tool requests with simplified API."""
