@@ -11,8 +11,7 @@ Features:
 
 import os
 import re
-from typing import Optional, List, Dict, Any, Union
-from pydantic import BaseModel
+from typing import Optional, List, Dict, Any
 
 try:
     import tiktoken
@@ -21,18 +20,6 @@ except ImportError:
     HAS_TIKTOKEN = False
 
 
-class LineArgs(BaseModel):
-    """Arguments for line-based reading."""
-    lines_start: int                    # Starting line number (1-based)
-    lines_end: Optional[int] = None     # Ending line number (optional)
-    complete_lines: bool = False        # If True, use tiered approach for long lines
-
-
-class RegexArgs(BaseModel):
-    """Arguments for regex-based reading."""
-    regex: str                          # Regex pattern to search for
-
-
 class ReadTool:
     """Simplified file reading tool optimized for AI agents."""
 
@@ -57,31 +44,48 @@ class ReadTool:
     def read_file(
         self,
         file_path: str,
-        read_args: Optional[Union[LineArgs, RegexArgs]] = None
+        lines_start: Optional[int] = None,
+        lines_end: Optional[int] = None,
+        regex: Optional[str] = None
     ) -> Dict[str, Any]:
         """
-        Read file content with smart token management and flexible reading modes.
-
-        USAGE:
-        - Full read: Small-medium files, initial exploration
-        - Line ranges: Specific sections, error locations (LineArgs)
-        - Regex search: Find patterns, function definitions (RegexArgs)
-
-        EXAMPLES:
-        read_file('src/main.py')  # Full file read
-        read_file('config.json', LineArgs(lines_start=10, lines_end=20))  # Line range
-        read_file('*.py', RegexArgs(regex=r'def main'))  # Regex search
-
-        LIMITATIONS & ERRORS:
-        - Large files (>50MB): Use line ranges to avoid token limits
-        - "File not found": Check path spelling and existence
-        - "Content truncated": File too large, use targeted reading
+        Read file with optional line range and/or regex filtering.
 
         Args:
-            file_path: absolute Path to file to read
-            read_args: Either LineArgs or RegexArgs for specific reading modes
+            file_path: Absolute path to file (e.g., /workspace/main.py)
+            lines_start: Start line number (1-based, optional)
+            lines_end: End line number (optional, requires lines_start)
+            regex: Regex pattern for search (case-insensitive, optional)
+
+        Reading Modes:
+            1. Full file: read_file('/workspace/config.json')
+            2. Line range: read_file('/workspace/main.py', lines_start=100, lines_end=200)
+            3. From line to end: read_file('/workspace/main.py', lines_start=200)
+            4. Regex search: read_file('/workspace/main.py', regex=r'def \w+')
+            5. Regex in range: read_file('/workspace/main.py', lines_start=100, lines_end=200, regex='import')
+               → Searches regex within line range, shows matches with context, reports outside matches
+
+        Common Patterns:
+            - Find functions: regex=r'def \w+'
+            - Find classes: regex=r'class \w+'
+            - Find imports: regex='import|from'
+            - Find errors: regex='error|exception|raise'
+
+        Important Notes:
+            - Line numbers are 1-based (first line is 1, not 0)
+            - Regex shows 2 lines of context before/after each match (marked with >>>)
+            - Context lines can extend outside specified line range
+            - Feedback always appended to output with >> prefix
+
         Returns:
-            Dict with content, metadata, and clear feedback
+            {
+                "success": true/false,
+                "content": "numbered file content",
+                "output": "content + feedback appended",
+                "total_lines": 100,
+                "lines_shown": 50,
+                "token_count": 1234
+            }
         """
         try:
             # Basic file validation
@@ -107,11 +111,16 @@ class ReadTool:
             if isinstance(lines, dict):  # Error response
                 return lines
 
-            # Route to appropriate processing
-            if isinstance(read_args, RegexArgs):
-                return self._process_regex_read(file_path, lines, read_args)
-            elif isinstance(read_args, LineArgs):
-                return self._process_line_read(file_path, lines, read_args)
+            # Route to appropriate processing based on parameters
+            if lines_start is not None and regex is not None:
+                # Combined mode: regex search within line range
+                return self._process_regex_in_range(file_path, lines, lines_start, lines_end, regex)
+            elif lines_start is not None:
+                # Line range mode
+                return self._process_line_read(file_path, lines, lines_start, lines_end)
+            elif regex is not None:
+                # Full file regex search
+                return self._process_regex_read(file_path, lines, regex)
             else:
                 # Full file read
                 return self._process_full_read(file_path, lines)
@@ -278,30 +287,51 @@ class ReadTool:
         limits = self._get_token_limits(lines)
         result = self._process_lines_with_limits(lines, 1, None, limits)
         result["file_path"] = file_path
+
+        # Add feedback
+        feedback = f">> Full file ({result['lines_shown']} lines, {result['token_count']} tokens)"
+        result["output"] = result.get("content", "") + f"\n{feedback}"
         return result
 
-    def _process_line_read(self, file_path: str, lines: List[str], args: LineArgs) -> Dict[str, Any]:
-        """Process line-based read with LineArgs."""
-        if args.complete_lines:
-            # Use tiered approach - always use 10k mode for complete lines
-            limits = {
-                "total_limit": 10000,
-                "per_line_limit": 6000,
-                "soft_buffer": 0
-            }
+    def _process_line_read(self, file_path: str, lines: List[str], lines_start: int, lines_end: Optional[int]) -> Dict[str, Any]:
+        """Process line-based read."""
+        total_lines = len(lines)
+
+        # Validate line range
+        if lines_start < 1:
+            feedback = f">> Invalid: lines_start={lines_start} must be >= 1. Showing from line 1"
+            lines_start = 1
+        elif lines_start > total_lines:
+            feedback = f">> Invalid: lines_start={lines_start} exceeds file length ({total_lines} lines). Showing full file"
+            lines_start = 1
+            lines_end = None
         else:
-            limits = self._get_token_limits(lines)
+            feedback = None
 
-        result = self._process_lines_with_limits(lines, args.lines_start, args.lines_end, limits)
+        if lines_end is not None and lines_end < lines_start:
+            feedback = f">> Invalid: lines_end={lines_end} < lines_start={lines_start}. Showing from line {lines_start} to end"
+            lines_end = None
+
+        limits = self._get_token_limits(lines)
+        result = self._process_lines_with_limits(lines, lines_start, lines_end, limits)
         result["file_path"] = file_path
+
+        # Append feedback to output
+        if feedback:
+            result["output"] = result.get("content", "") + f"\n{feedback}"
+        else:
+            actual_end = lines_end if lines_end else total_lines
+            feedback = f">> Lines {lines_start}-{min(actual_end, result['end_line'])} ({result['lines_shown']} lines, {result['token_count']} tokens)"
+            result["output"] = result.get("content", "") + f"\n{feedback}"
+
         return result
 
-    def _process_regex_read(self, file_path: str, lines: List[str], args: RegexArgs) -> Dict[str, Any]:
-        """Process regex-based read with RegexArgs."""
+    def _process_regex_read(self, file_path: str, lines: List[str], regex_pattern: str) -> Dict[str, Any]:
+        """Process regex-based read on full file."""
         try:
             # Compile regex pattern
             try:
-                regex = re.compile(args.regex, re.IGNORECASE | re.MULTILINE)
+                regex = re.compile(regex_pattern, re.IGNORECASE | re.MULTILINE)
             except re.error as e:
                 return {
                     "success": False,
@@ -317,14 +347,15 @@ class ReadTool:
             matching_lines = sorted(matching_lines)  # Sort by line number
 
             if not matching_lines:
+                feedback = f">> Regex '{regex_pattern}': no matches found in file ({len(lines)} lines)"
                 return {
                     "success": True,
                     "content": "",
+                    "output": feedback,
                     "file_path": file_path,
-                    "pattern": args.regex,
+                    "pattern": regex_pattern,
                     "total_matches": 0,
-                    "matches_shown": 0,
-                    "message": f"No matches found for pattern '{args.regex}'"
+                    "matches_shown": 0
                 }
 
             # Determine if we need two-phase approach
@@ -339,19 +370,20 @@ class ReadTool:
 
             if all_tokens <= max_allowed:
                 # Phase 1 only - all matches fit
+                feedback = f">> Regex '{regex_pattern}': {len(matching_lines)} matches found, all shown with context"
                 return {
                     "success": True,
                     "content": all_matches_content,
+                    "output": all_matches_content + f"\n{feedback}",
                     "file_path": file_path,
-                    "pattern": args.regex,
+                    "pattern": regex_pattern,
                     "total_matches": len(matching_lines),
                     "matches_shown": len(matching_lines),
-                    "token_count": all_tokens,
-                    "message": f"Found {len(matching_lines)} matches, all shown with context"
+                    "token_count": all_tokens
                 }
             else:
                 # Two-phase approach
-                return self._two_phase_regex_read(file_path, lines, matching_lines, args.regex, limits)
+                return self._two_phase_regex_read(file_path, lines, matching_lines, regex_pattern, limits)
 
         except Exception as e:
             return {
@@ -441,9 +473,11 @@ class ReadTool:
         content = '\n'.join(all_content)
         total_tokens = phase1_tokens + phase2_tokens
 
+        feedback = f">> Regex '{pattern}': {len(matching_lines)} matches, showing {len(phase1_matches)} with context + {len(phase2_content)} line references"
         return {
             "success": True,
             "content": content,
+            "output": content + f"\n{feedback}",
             "file_path": file_path,
             "pattern": pattern,
             "total_matches": len(matching_lines),
@@ -451,10 +485,91 @@ class ReadTool:
             "phase1_matches": len(phase1_matches),
             "phase2_matches": len(phase2_content),
             "remaining_matches": remaining,
-            "token_count": total_tokens,
-            "message": f"Found {len(matching_lines)} matches, showing {len(phase1_matches)} with context + {len(phase2_content)} line references"
+            "token_count": total_tokens
         }
 
+    def _process_regex_in_range(self, file_path: str, lines: List[str], lines_start: int,
+                                lines_end: Optional[int], regex_pattern: str) -> Dict[str, Any]:
+        """Process regex search within a specific line range."""
+        try:
+            # Compile regex pattern
+            try:
+                regex = re.compile(regex_pattern, re.IGNORECASE | re.MULTILINE)
+            except re.error as e:
+                return {
+                    "success": False,
+                    "error": f"Invalid regex pattern: {str(e)}"
+                }
+
+            total_lines = len(lines)
+            actual_end = lines_end if lines_end else total_lines
+
+            # Find matches in entire file
+            all_matches = set()
+            for line_num, line in enumerate(lines, 1):
+                if regex.search(line):
+                    all_matches.add(line_num)
+
+            # Separate matches inside and outside range
+            matches_in_range = sorted([ln for ln in all_matches if lines_start <= ln <= actual_end])
+            matches_outside_range = sorted([ln for ln in all_matches if ln < lines_start or ln > actual_end])
+
+            # Build content with context for matches in range
+            if matches_in_range:
+                content = self._build_regex_matches(lines, matches_in_range, context_lines=2)
+                tokens = self.count_tokens(content)
+
+                # Build feedback
+                if matches_outside_range:
+                    outside_sample = matches_outside_range[:7]  # Show up to 7 line numbers
+                    sample_str = ", ".join(str(ln) for ln in outside_sample)
+                    if len(matches_outside_range) > 7:
+                        sample_str += f", ... ({len(matches_outside_range) - 7} more)"
+                    feedback = f">> Regex '{regex_pattern}' in lines {lines_start}-{actual_end}: {len(matches_in_range)} matches found. {len(matches_outside_range)} additional matches outside range at lines: {sample_str}"
+                else:
+                    feedback = f">> Regex '{regex_pattern}' in lines {lines_start}-{actual_end}: {len(matches_in_range)} matches found"
+
+                return {
+                    "success": True,
+                    "content": content,
+                    "output": content + f"\n{feedback}",
+                    "file_path": file_path,
+                    "pattern": regex_pattern,
+                    "total_matches": len(all_matches),
+                    "matches_shown": len(matches_in_range),
+                    "matches_in_range": len(matches_in_range),
+                    "matches_outside_range": len(matches_outside_range),
+                    "token_count": tokens
+                }
+            else:
+                # No matches in range
+                if matches_outside_range:
+                    outside_sample = matches_outside_range[:10]  # Show up to 10 line numbers
+                    sample_str = ", ".join(str(ln) for ln in outside_sample)
+                    if len(matches_outside_range) > 10:
+                        sample_str += f", ..."
+                    feedback = f">> Regex '{regex_pattern}' in lines {lines_start}-{actual_end}: no matches found. However, found {len(matches_outside_range)} matches outside this range at lines: {sample_str}"
+                else:
+                    feedback = f">> Regex '{regex_pattern}' in lines {lines_start}-{actual_end}: no matches found in entire file"
+
+                return {
+                    "success": True,
+                    "content": "",
+                    "output": feedback,
+                    "file_path": file_path,
+                    "pattern": regex_pattern,
+                    "total_matches": len(all_matches),
+                    "matches_shown": 0,
+                    "matches_in_range": 0,
+                    "matches_outside_range": len(matches_outside_range)
+                }
+
+        except Exception as e:
+            return {
+                "success": False,
+                "error": f"Failed to process regex in range: {str(e)}"
+            }
+
     def _build_single_match(self, lines: List[str], line_num: int, context_lines: int) -> str:
         """Build formatted output for a single regex match with context."""
         start_ctx = max(1, line_num - context_lines)
