@@ -89,32 +89,117 @@ class RunTool:
         interactive: bool = False
     ) -> Dict[str, Any]:
         """
-        Execute commands with 2-state management: IDLE (ready) or RUNNING (active process).
+        Execute shell commands in the Docker workspace environment.
 
-        SUCCESS DEFINITION:
-        - success=True: Command was executed (regardless of exit code)
-        - success=False: Tool error (timeout, cleanup failed, invalid state, etc.)
+        SUCCESS MEANING:
+        - success=True: Command executed (exit code 0 or non-zero doesn't matter)
+        - success=False: Tool error (timeout, invalid state, process management issue)
 
-        USAGE:
-        - Non-interactive: Quick commands, auto-kills on timeout
-        - Interactive: Long tasks needing input, returns control choices on timeout
-        - Control commands when RUNNING: '' (continue), 'C-c' (kill), '>>> input' (send input)
+        MODES:
+
+        NON-INTERACTIVE (default, interactive=False):
+        - Use for: Quick commands that complete in seconds
+        - Behavior: Runs command, waits for completion (up to timeout)
+        - On timeout: Automatically kills process and returns partial output
+        - Examples: ls, cat, grep, python script.py, npm install
+
+        INTERACTIVE (interactive=True):
+        - Use for: Long-running commands, training loops, servers
+        - Behavior: Runs command, returns control after timeout with options
+        - On timeout: Keeps process running, you can continue/kill/send input
+        - Examples: python train.py, npm start, watch mode commands
 
         EXAMPLES:
-        run_command('npm test', timeout=60, interactive=False)  # Normal command
-        run_command('C-c')  # Control: kill running process
-        run_command('>>> 5')  # Control: send 5 as input to the program
 
-        COMMON ERRORS:
-        - "Another command running": Use 'C-c' to kill or wait for completion
-        - "Command timed out": Process killed after timeout in non-interactive mode
+        # Basic commands (non-interactive)
+        run_command('ls -la /workspace')
+        run_command('python test.py', timeout=60)
+        run_command('cat /workspace/config.json')
+        run_command('grep -r "TODO" /workspace', timeout=30)
+
+        # Long-running commands (interactive)
+        run_command('python train.py --epochs 100', timeout=300, interactive=True)
+        # ... after timeout, returns with control options ...
+        run_command('')  # Continue for another timeout period
+        run_command('C-c')  # Kill the process
+
+        # Commands with input (interactive)
+        run_command('python -c "x = input(\\'Enter: \\'); print(x)"',
+                    timeout=30, interactive=True)
+        # ... after timeout ...
+        run_command('>>> hello')  # Send "hello" as input
+
+        # Install packages
+        run_command('pip install numpy pandas', timeout=120)
+        run_command('npm install', timeout=180)
+
+        # File operations
+        run_command('mkdir -p /workspace/data')
+        run_command('cp source.txt dest.txt')
+        run_command('rm -rf /tmp/cache')
+
+        # Git operations
+        run_command('git status')
+        run_command('git diff HEAD~1')
+
+        CONTROL COMMANDS (use after interactive timeout):
+        - '' (empty string): Continue process for another timeout period
+        - 'C-c': Kill the running process immediately
+        - '>>> text': Send "text" as input to the process (like typing and pressing Enter)
+
+        IMPORTANT NOTES:
+        - Commands run in Docker container at /workspace directory
+        - Timeout in seconds (default 300 = 5 minutes)
+        - Output includes both stdout and stderr combined
+        - Exit code is prepended to output (e.g., "Exit code: 0")
+        - Process is fully cleaned up after completion/timeout/kill
+        - Only ONE command can run at a time
+
+        CHOOSING TIMEOUT:
+        - Quick commands (ls, cat, grep): 10-30 seconds
+        - Package installs (pip, npm): 120-300 seconds
+        - Tests/builds: 60-600 seconds
+        - Training/long tasks: 300-3600 seconds (use interactive=True)
+
+        UNDERSTANDING OUTPUT:
+        The "output" field contains:
+        - Exit code (e.g., "Exit code: 0" or "Exit code: 1")
+        - Combined stdout and stderr
+
+        The "message" field contains:
+        - Execution time (e.g., "executed in 2.5sec")
+        - Process status (e.g., "Process finished and cleaned")
+        - Control suggestions if interactive timeout
+
+        RETURN FORMAT:
+        {
+            "success": true/false,
+            "output": "Exit code: 0\\ncommand output here...",
+            "message": "executed in 5.2sec\\nProcess finished and cleaned"
+        }
+
+        COMMON PATTERNS:
+        # Check if file exists
+        run_command('test -f /workspace/file.txt && echo "exists" || echo "not found"')
+
+        # Count lines in file
+        run_command('wc -l /workspace/data.csv')
+
+        # Find files
+        run_command('find /workspace -name "*.py"')
+
+        # Run tests
+        run_command('pytest tests/', timeout=120)
+
+        # Start training (interactive)
+        run_command('python train.py', timeout=600, interactive=True)
 
         Args:
-            command: Command to execute or control command ('', 'C-c', '>>> input')
-            timeout: Maximum execution time in seconds (default: 300)
-            interactive: If True, wait for timeout and return control choices
+            command: Shell command to execute or control command ('', 'C-c', '>>> input')
+            timeout: Maximum time in seconds before returning control (default: 300)
+            interactive: False=auto-kill on timeout, True=return control with options
         Returns:
-            Dict with simplified output format: success, output, message
+            Dict with success, output (with exit code), and execution message
         """
         # Step 1: Check current state
         current_state = self._get_state()
